

facts - senan_db
  det(string, string)  		% det(determ, type)
  noun(string)
  rel(string)
  verb(string)
  desc(string, string)       	% det(noun, type)
  ant(string, string)		% det(ant1, ant2)
domains
  DETERM	= none;
  		  determ(string)
  NOUNP		= nounp(DETERM,DETERM,string,RELCL)
  RELCL		= none;
  		  relcl(string,VERBP)
  SENTENCE	= sent(NOUNP,VERBP)
  VERBP		= verb(string);
		  verbp(string,NOUNP)
  TOKL		= string*

predicates
% Recognition of words in different forms
  is_det(string)
  is_noun(string)
  is_rel(string)
  is_verb(string)


% Parser
  nondeterm s_determ(TOKL,TOKL,DETERM)
  nondeterm s_nounp(TOKL,TOKL,NOUNP)
  nondeterm s_relcl(TOKL,TOKL,RELCL)
  nondeterm s_sentence(TOKL,TOKL,SENTENCE)
  nondeterm s_verbp(TOKL,TOKL,VERBP)

  nondeterm is_related(string, DETERM)
  nondeterm is_sametype(DETERM,DETERM)  

% scanner
  check(string)
  tokl(string,TOKL)

clauses
  s_sentence(TOKL,TOKL2,sent(NOUNP,VERBP)):-
	s_nounp(TOKL,TOKL1,NOUNP),
	s_verbp(TOKL1,TOKL2,VERBP),
	TOKL2=[],
	!.
  s_sentence(_,_,_):-
	write(">> Sentence not recognized\n"),
	fail.

  s_nounp(TOKL,TOKL3,nounp(DETERM,DETERM2,NOUN,RELCL)):-
	s_determ(TOKL,TOKL1,DETERM),
	s_determ(TOKL1,[NOUN|TOKL2],DETERM2),
	is_noun(NOUN),
% Check on semantic bond
	
	is_related(NOUN,DETERM),
	is_related(NOUN,DETERM2),
	is_sametype(DETERM,DETERM2),	
	
	s_relcl(TOKL2,TOKL3,RELCL).

  is_related(NOUN, none):- !.
  is_related(NOUN, determ(DETERM)):-
  	det(DETERM, TYPE),
  	desc(NOUN, TYPE),
  	!.
  is_related(NOUN, determ(DETERM)):-
	det(DETERM, T),
	write(">> Meaning is out of thesaurus -> \"",NOUN,"\" is not specifyed by \"",T,"\"\n"),
	exit.  	
  is_sametype(DETERM, none):- !.
  is_sametype(none, DETERM):- !.
  is_sametype(none, none):- !.
  is_sametype(determ(DETERM),determ(DETERM2)):-
  	det(DETERM, TYPE),
  	det(DETERM2, TYPE),
  	!,
  	write(">> Determinants of the same type -> Different \"",TYPE,"\"s can't be determinants of one object \n"),
  	exit.

  is_sametype(determ(DETERM),determ(DETERM2)):-
  	det(DETERM, T1),
  	det(DETERM2, T2).

  s_determ([DETERM|TOKL],TOKL,determ(DETERM)):-
	is_det(DETERM).
  s_determ(TOKL,TOKL,none).

  s_relcl([REL|TOKL],TOKL1,relcl(REL,VERBP)):-
	is_rel(REL),
	s_verbp(TOKL,TOKL1,VERBP).
  s_relcl(TOKL,TOKL,none).

  s_verbp([VERB|TOKL],TOKL1,verbp(VERB,NOUNP)):-
	is_verb(VERB),
	s_nounp(TOKL,TOKL1,NOUNP).
  s_verbp([VERB|TOKL],TOKL,verb(VERB)):-
	is_verb(VERB).

  is_noun(X):-
	noun(X),
	!.
  is_noun(X):-
	noun(Y),
	concat(Y,"s",X),
	!.

  is_det(X):-
	det(X,_),
	!.

  is_rel(X):-
	rel(X),
	!.

  is_verb(X):-
	verb(X),
	!.
  is_verb(X):-
	verb(Y),
	concat(Y,"s",X),
	!.
  is_verb(X):-
	verb(Y),
	concat(Y,"ed",X),
	!.
  is_verb(X):-
	verb(Y),
	concat(Y,"es",X),
	!.
  is_verb(X):-
	verb(Y),
	concat(Y,"ing",X),
	!.

  tokl(STR,[TOK|TOKL]):-
	fronttoken(STR,TOK,STR1),
	check(TOK),
	!,
	tokl(STR1,TOKL).
  tokl(_,[]).

  check(WORD):-
	is_noun(WORD),
	!.
  check(WORD):-
	is_det(WORD),
	!.
  check(WORD):-
	is_rel(WORD),
	!.
  check(WORD):-
	is_verb(WORD),
	!.
  check(WORD):-
	write(">> Unknown word: ",WORD,"\n"),
	fail.

goal
  consult("sen_an.dba",senan_db),
  write("Try: black cat love easy complex Prolog\n"),
  write("Write a sentence: "),
  readln(STR),
  tokl(STR,TOKL),
  s_sentence(TOKL,RESTTOKL,SENT),
  RESTTOKL=[],
  write(SENT).
